@@ -21,10 +21,11 @@
 #ifndef vte_vtedraw_h_included
 #define vte_vtedraw_h_included
 
-#ident "$Id: vtedraw.h,v 1.3 2003/04/14 19:41:12 nalin Exp $"
+#ident "$Id: vtedraw.h,v 1.6 2003/05/13 04:27:39 nalin Exp $"
 
 #include <glib.h>
 #include <gtk/gtk.h>
+#include "vtebg.h"
 
 G_BEGIN_DECLS
 
@@ -57,7 +58,13 @@ struct _vte_draw_impl {
 	void (*start)(struct _vte_draw *draw);
 	void (*end)(struct _vte_draw *draw);
 	void (*set_background_color)(struct _vte_draw *, GdkColor *);
-	void (*set_background_pixbuf)(struct _vte_draw *, GdkPixbuf *);
+	void (*set_background_image)(struct _vte_draw *,
+				     enum VteBgSourceType type,
+				     GdkPixbuf *pixbuf,
+				     const char *file,
+				     const GdkColor *color,
+				     double saturation);
+	gboolean requires_repaint;
 	void (*clear)(struct _vte_draw *, gint, gint, gint, gint);
 	void (*set_text_font)(struct _vte_draw *, const PangoFontDescription *);
 	int (*get_text_width)(struct _vte_draw *);
@@ -92,7 +99,8 @@ void _vte_draw_free(struct _vte_draw *draw);
    implementations may require that this visual/colormap pair be used when
    creating a window, and may fail otherwise. */
 GdkVisual *_vte_draw_get_visual(struct _vte_draw *draw);
-GdkColormap *_vte_draw_get_colormap(struct _vte_draw *draw);
+GdkColormap *_vte_draw_get_colormap(struct _vte_draw *draw,
+				    gboolean maybe_use_default);
 
 /* Begin and end a drawing operation.  If anything is buffered locally, it is
    flushed to the window system when _end() is called. */
@@ -102,7 +110,13 @@ void _vte_draw_end(struct _vte_draw *draw);
 /* Set the background color, a background pixbuf (if you want transparency,
    you'll have to do that yourself), and clear an area to the default. */
 void _vte_draw_set_background_color(struct _vte_draw *draw, GdkColor *color);
-void _vte_draw_set_background_pixbuf(struct _vte_draw *draw, GdkPixbuf *pixbuf);
+void _vte_draw_set_background_image(struct _vte_draw *draw,
+				    enum VteBgSourceType type,
+				    GdkPixbuf *pixbuf,
+				    const char *file,
+				    const GdkColor *color,
+				    double saturation);
+gboolean _vte_draw_requires_repaint(struct _vte_draw *draw);
 void _vte_draw_clear(struct _vte_draw *draw,
 		     gint x, gint y, gint width, gint height);
 
