@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: vtegl.c,v 1.8 2003/05/06 08:14:49 nalin Exp $"
+#ident "$Id: vtegl.c,v 1.11 2003/05/13 04:27:39 nalin Exp $"
 
 #include "../config.h"
 
@@ -35,6 +35,7 @@
 #include <GL/glu.h>
 #include <GL/glx.h>
 #include <X11/Xutil.h>
+#include "buffer.h"
 #include "debug.h"
 #include "vtedraw.h"
 #include "vtefc.h"
@@ -46,24 +47,19 @@ struct _vte_gl_data
 	XVisualInfo *visual_info;
 	GLXContext context;
 	GdkColor color;
-	GdkPixmap *bgpixmap, *pixmap;
-	GLXPixmap glpixmap;
+	GdkPixbuf *bgpixbuf;
+	GLXDrawable glwindow;
 	gint scrollx, scrolly;
 	struct _vte_glyph_cache *cache;
+	struct _vte_buffer *buffer;
 };
 
-#define _vte_gl_pixmap_attributes \
-	GLX_RGBA, \
-	GLX_RED_SIZE, 8, \
-	GLX_BLUE_SIZE, 8, \
-	GLX_GREEN_SIZE, 8, \
-	GLX_ALPHA_SIZE, 8, \
-	None,
+#define _vte_gl_attributes GLX_USE_GL, GLX_DOUBLEBUFFER, GLX_RGBA, None,
 
 static gboolean
 _vte_gl_check(struct _vte_draw *draw, GtkWidget *widget)
 {
-	int pixmap_attributes[] = {_vte_gl_pixmap_attributes};
+	int attributes[] = {_vte_gl_attributes};
 	XVisualInfo *visual_info;
 	GLXContext context = NULL;
 	GdkDisplay *gdisplay;
@@ -71,6 +67,7 @@ _vte_gl_check(struct _vte_draw *draw, GtkWidget *widget)
 	GdkScreen *gscreen;
 	int screen;
 	int error, event;
+	gboolean direct;
 
 	gdisplay = gdk_display_get_default();
 	display = gdk_x11_display_get_xdisplay(gdisplay);
@@ -86,8 +83,7 @@ _vte_gl_check(struct _vte_draw *draw, GtkWidget *widget)
 	}
 
 	/* See if a suitable visual exists. */
-	visual_info = glXChooseVisual(display, screen,
-				      pixmap_attributes);
+	visual_info = glXChooseVisual(display, screen, attributes);
 	if (visual_info == NULL) {
 #ifdef VTE_DEBUG
 		g_warning("Unable to find a suitable GLX visual.\n");
@@ -96,27 +92,36 @@ _vte_gl_check(struct _vte_draw *draw, GtkWidget *widget)
 	}
 
 	/* Create a GLX context. */
-	context = glXCreateContext(display, visual_info, NULL, False);
+	context = glXCreateContext(display, visual_info, NULL, GL_TRUE);
 	if (context == NULL) {
 #ifdef VTE_DEBUG
 		g_warning("Unable to create a GLX context.\n");
 #endif
 		return FALSE;
 	}
+
+	/* Check if it's a direct rendering context. */
+	direct = glXIsDirect(display, context);
+#ifdef VTE_DEBUG
+	if (!direct) {
+		g_warning("Unable to create a direct GLX context.\n");
+	}
+#endif
 	glXDestroyContext(display, context);
 
-	return TRUE;
+	return (direct == True) ? TRUE : FALSE;
 }
 
 static void
 _vte_gl_create(struct _vte_draw *draw, GtkWidget *widget)
 {
 	struct _vte_gl_data *data;
-	int pixmap_attributes[] = {_vte_gl_pixmap_attributes};
+	int attributes[] = {_vte_gl_attributes};
 	GdkDisplay *gdisplay;
 	Display *display;
 	GdkScreen *gscreen;
 	int screen;
+	gboolean direct;
 
 	draw->impl_data = g_malloc(sizeof(struct _vte_gl_data));
 	data = (struct _vte_gl_data*) draw->impl_data;
@@ -126,25 +131,32 @@ _vte_gl_create(struct _vte_draw *draw, GtkWidget *widget)
 	gscreen = gdk_screen_get_default();
 	screen = gdk_x11_screen_get_screen_number(gscreen);
 
-	data->visual_info = glXChooseVisual(display, screen,
-					    pixmap_attributes);
+	data->visual_info = glXChooseVisual(display, screen, attributes);
 	if (data->visual_info == NULL) {
-		g_error("Unable to select GLX visual.\n");
+		g_error("Unable to find a suitable GLX visual.\n");
 	}
+
 	data->context = glXCreateContext(display, data->visual_info,
-					 NULL, False);
+					 NULL, GL_TRUE);
 	if (data->context == NULL) {
-		g_error("Unable to create GLX context.\n");
+		g_error("Unable to create a GLX context.\n");
+	}
+
+	direct = glXIsDirect(display, data->context);
+	if (!direct) {
+		g_error("Unable to create a direct GLX context.\n");
 	}
 
 	data->color.red = 0;
 	data->color.green = 0;
 	data->color.blue = 0;
-	data->bgpixmap = NULL;
-	data->pixmap = NULL;
-	data->glpixmap = -1;
+	data->bgpixbuf = NULL;
+	data->glwindow = -1;
 	data->scrollx = data->scrolly = 0;
 	data->cache = _vte_glyph_cache_new();
+	data->buffer = _vte_buffer_new();
+
+	gtk_widget_set_double_buffered(widget, FALSE);
 }
 
 static void
@@ -162,23 +174,18 @@ _vte_gl_destroy(struct _vte_draw *draw)
 	gscreen = gdk_screen_get_default();
 	screen = gdk_x11_screen_get_screen_number(gscreen);
 
+	_vte_buffer_free(data->buffer);
+	data->buffer = NULL;
+
 	_vte_glyph_cache_free(data->cache);
 	data->cache = NULL;
 
-	if (data->glpixmap != -1) {
-		glXDestroyGLXPixmap(display, data->glpixmap);
+	if (GDK_IS_PIXBUF(data->bgpixbuf)) {
+		g_object_unref(G_OBJECT(data->bgpixbuf));
 	}
-	data->glpixmap = -1;
+	data->bgpixbuf = NULL;
 
-	if (GDK_IS_PIXBUF(data->pixmap)) {
-		g_object_unref(G_OBJECT(data->pixmap));
-	}
-	data->pixmap = NULL;
-
-	if (GDK_IS_PIXBUF(data->bgpixmap)) {
-		g_object_unref(G_OBJECT(data->bgpixmap));
-	}
-	data->bgpixmap = NULL;
+	glXMakeCurrent(display, None, data->context);
 
 	glXDestroyContext(display, data->context);
 	data->context = NULL;
@@ -198,6 +205,10 @@ _vte_gl_get_visual(struct _vte_draw *draw)
 
 	data = (struct _vte_gl_data*) draw->impl_data;
 	gscreen = gdk_screen_get_default();
+#ifdef VTE_DEBUG
+	g_print("Using GLX-capable visual 0x%02lx.\n",
+		(long) data->visual_info->visualid);
+#endif
 	return gdk_x11_screen_lookup_visual(gscreen,
 					    data->visual_info->visualid);
 }
@@ -214,33 +225,22 @@ _vte_gl_start(struct _vte_draw *draw)
 	struct _vte_gl_data *data;
 	GdkDisplay *gdisplay;
 	Display *display;
-	gint width, height, depth;
+	gint width, height;
 
 	data = (struct _vte_gl_data*) draw->impl_data;
 	gdisplay = gdk_display_get_default();
 	display = gdk_x11_display_get_xdisplay(gdisplay);
 
-	if (data->glpixmap != -1) {
-		glXDestroyGLXPixmap(display, data->glpixmap);
-		data->glpixmap = -1;
-	}
-	if (GDK_IS_PIXMAP(data->pixmap)) {
-		g_object_unref(G_OBJECT(data->pixmap));
-	}
-
-	width = height = depth = 0;
+	data->glwindow = gdk_x11_drawable_get_xid(draw->widget->window);
 	gdk_drawable_get_size(draw->widget->window, &width, &height);
-	depth = gdk_drawable_get_depth(draw->widget->window);
-	data->pixmap = gdk_pixmap_new(draw->widget->window,
-				      width, height, depth);
-	data->glpixmap = glXCreateGLXPixmap(display, data->visual_info,
-					    GDK_WINDOW_XID(data->pixmap));
 
-	glXMakeCurrent(display, data->glpixmap, data->context);
+	glXMakeCurrent(display, data->glwindow, data->context);
 	glMatrixMode(GL_PROJECTION);
 	glLoadIdentity();
-        gluOrtho2D(0, width, 0, height);
-        glViewport(0, 0, width, height);
+	gluOrtho2D(0, width - 1, height - 1, 0);
+	glViewport(0, height - 1, width, -height);
+	glEnable(GL_BLEND);
+	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 }
 
 static void
@@ -249,40 +249,22 @@ _vte_gl_end(struct _vte_draw *draw)
 	struct _vte_gl_data *data;
 	GdkDisplay *gdisplay;
 	Display *display;
-	int state, width, height;
 
 	data = (struct _vte_gl_data*) draw->impl_data;
 	gdisplay = gdk_display_get_default();
 	display = gdk_x11_display_get_xdisplay(gdisplay);
 
-	glFlush();
-	glXWaitX();
-	glXWaitGL();
+	glXMakeCurrent(display, data->glwindow, data->context);
+	glXSwapBuffers(display, data->glwindow);
 
-	if (GDK_IS_PIXMAP(data->pixmap)) {
-		state = GTK_WIDGET_STATE(draw->widget);
-		width = height = 0;
-		gdk_drawable_get_size(data->pixmap, &width, &height);
-		gdk_draw_drawable(draw->widget->window,
-				  draw->widget->style->fg_gc[state],
-				  data->pixmap,
-				  0, 0, 0, 0,
-				  width, height);
-	}
-	if (data->glpixmap != -1) {
-		glXDestroyGLXPixmap(display, data->glpixmap);
-		data->glpixmap = -1;
-	}
-	if (GDK_IS_PIXMAP(data->pixmap)) {
-		g_object_unref(G_OBJECT(data->pixmap));
-	}
-	data->pixmap = NULL;
+	data->glwindow = -1;
 }
 
 static void
 _vte_gl_set_background_color(struct _vte_draw *draw, GdkColor *color)
 {
 	struct _vte_gl_data *data;
+
 	data = (struct _vte_gl_data*) draw->impl_data;
 	data->color = *color;
 }
@@ -296,72 +278,97 @@ _vte_gl_set_background_image(struct _vte_draw *draw,
 			     double saturation)
 {
 	struct _vte_gl_data *data;
-	GdkPixmap *bgpixmap;
+	GdkPixbuf *bgpixbuf;
 
 	data = (struct _vte_gl_data*) draw->impl_data;
-	bgpixmap = vte_bg_get_pixmap(vte_bg_get(), type, pixbuf, file,
-				     tint, saturation,
-				     _vte_draw_get_colormap(draw));
-	if (GDK_IS_PIXMAP(data->bgpixmap)) {
-		g_object_unref(G_OBJECT(data->bgpixmap));
+	bgpixbuf = vte_bg_get_pixbuf(vte_bg_get(), type, pixbuf, file,
+				     tint, saturation);
+	if (GDK_IS_PIXBUF(data->bgpixbuf)) {
+		g_object_unref(G_OBJECT(data->bgpixbuf));
 	}
-	data->bgpixmap = bgpixmap;
+	data->bgpixbuf = bgpixbuf;
 }
 
 static void
-_vte_gl_clear(struct _vte_draw *draw,
-		 gint x, gint y, gint width, gint height)
+_vte_gl_clear(struct _vte_draw *draw, gint x, gint y, gint width, gint height)
 {
+	GdkDisplay *gdisplay;
+	Display *display;
 	struct _vte_gl_data *data;
-	data = (struct _vte_gl_data*) draw->impl_data;
 	long xstop, ystop, i, j;
-	int pixmapw, pixmaph, w, h;
+	int pixbufw, pixbufh, w, h, channels, stride;
+	GLenum format = 0;
+	guchar *pixels;
 
-	if (GDK_IS_PIXMAP(data->bgpixmap)) {
-		gdk_drawable_get_size(data->bgpixmap, &pixmapw, &pixmaph);
+	data = (struct _vte_gl_data*) draw->impl_data;
+	gdisplay = gdk_display_get_default();
+	display = gdk_x11_display_get_xdisplay(gdisplay);
+
+	glXMakeCurrent(display, data->glwindow, data->context);
+
+	if (GDK_IS_PIXBUF(data->bgpixbuf)) {
+		pixbufw = gdk_pixbuf_get_width(data->bgpixbuf);
+		pixbufh = gdk_pixbuf_get_height(data->bgpixbuf);
 	} else {
-		pixmapw = pixmaph = 0;
+		pixbufw = pixbufh = 0;
 	}
-	if ((pixmapw == 0) || (pixmaph == 0)) {
-		glXWaitX();
-		gdk_drawable_get_size(data->pixmap, &w, &h);
-		glBegin(GL_POLYGON);
-		glColor4us(data->color.red, data->color.green, data->color.blue,
+
+	if ((pixbufw == 0) || (pixbufh == 0)) {
+		glColor4us(data->color.red,
+			   data->color.green,
+			   data->color.blue,
 			   0xffff);
-		glVertex2d(x, h - y);
-		glVertex2d(x + width, h - y);
-		glVertex2d(x + width, h - (y + height));
-		glVertex2d(x, h - (y + height));
+		glBegin(GL_POLYGON);
+		glVertex2d(x, y);
+		glVertex2d(x + width, y);
+		glVertex2d(x + width, y + height - 1);
+		glVertex2d(x, y + height - 1);
 		glEnd();
 		return;
 	}
 
 	/* Flood fill. */
-	glXWaitGL();
-
 	xstop = x + width;
 	ystop = y + height;
 
+	pixels = gdk_pixbuf_get_pixels(data->bgpixbuf);
+	channels = gdk_pixbuf_get_n_channels(data->bgpixbuf);
+	stride = gdk_pixbuf_get_rowstride(data->bgpixbuf);
+
+	switch (channels) {
+	case 3:
+		format = GL_RGB;
+		break;
+	case 4:
+		format = GL_RGBA;
+		break;
+	default:
+		g_assert_not_reached();
+		break;
+	}
+
 	y = ystop - height;
-	j = (data->scrolly + y) % pixmaph;
+	j = (data->scrolly + y) % pixbufh;
 	while (y < ystop) {
 		x = xstop - width;
-		i = (data->scrollx + x) % pixmapw;
-		h = MIN(pixmaph - (j % pixmaph), ystop - y);
+		i = (data->scrollx + x) % pixbufw;
+
+		/* h = MIN(pixbufh - (j % pixbufh), ystop - y); */
+		h = 1;
 		while (x < xstop) {
-			w = MIN(pixmapw - (i % pixmapw), xstop - x);
-			gdk_draw_drawable(data->pixmap,
-					  draw->widget->style->fg_gc[GTK_WIDGET_STATE(draw->widget)],
-					  data->bgpixmap,
-					  i, j,
-					  x, y,
-					  w, h);
+			w = MIN(pixbufw - (i % pixbufw), xstop - x);
+
+			glRasterPos2i(x, y);
+			glDrawPixels(w, h,
+				     format, GL_UNSIGNED_BYTE,
+				     pixels + stride * j + channels * i);
 			x += w;
 			i = 0;
 		}
 		y += h;
-		j = 0;
+		j = (data->scrolly + y) % pixbufh;
 	}
+	glFlush();
 }
 
 static void
@@ -380,13 +387,14 @@ _vte_gl_set_text_font(struct _vte_draw *draw,
 		      const PangoFontDescription *fontdesc)
 {
 	struct _vte_gl_data *data;
-	data = (struct _vte_gl_data*) draw->impl_data;
 
+	data = (struct _vte_gl_data*) draw->impl_data;
 	if (data->cache != NULL) {
 		_vte_glyph_cache_free(data->cache);
 		data->cache = NULL;
 	}
 	data->cache = _vte_glyph_cache_new();
+
 	_vte_glyph_cache_set_font_description(NULL, data->cache, fontdesc,
 					      _vte_gl_fcpattern_disable_rgba,
 					      NULL);
@@ -396,6 +404,7 @@ static int
 _vte_gl_get_text_width(struct _vte_draw *draw)
 {
 	struct _vte_gl_data *data;
+
 	data = (struct _vte_gl_data*) draw->impl_data;
 	return data->cache->width;
 }
@@ -404,6 +413,7 @@ static int
 _vte_gl_get_text_height(struct _vte_draw *draw)
 {
 	struct _vte_gl_data *data;
+
 	data = (struct _vte_gl_data*) draw->impl_data;
 	return data->cache->height;
 }
@@ -412,6 +422,7 @@ static int
 _vte_gl_get_text_ascent(struct _vte_draw *draw)
 {
 	struct _vte_gl_data *data;
+
 	data = (struct _vte_gl_data*) draw->impl_data;
 	return data->cache->ascent;
 }
@@ -427,46 +438,104 @@ _vte_gl_draw_text(struct _vte_draw *draw,
 		  struct _vte_draw_text_request *requests, gsize n_requests,
 		  GdkColor *color, guchar alpha)
 {
+	GdkDisplay *gdisplay;
+	Display *display;
 	struct _vte_gl_data *data;
 	const struct _vte_glyph *glyph;
 	guint16 a, r, g, b;
-	int width, height;
-	int i, j, x, y, w, pad;
+	int i, j, x, y, w, pad, rows, columns, src, dest;
+	guchar *pixels;
 
 	data = (struct _vte_gl_data*) draw->impl_data;
+	gdisplay = gdk_display_get_default();
+	display = gdk_x11_display_get_xdisplay(gdisplay);
 
-	glXWaitX();
+	glXMakeCurrent(display, data->glwindow, data->context);
 
-	r = color->red;
-	g = color->green;
-	b = color->blue;
+	r = color->red >> 8;
+	g = color->green >> 8;
+	b = color->blue >> 8;
 
-	gdk_drawable_get_size(data->pixmap, &width, &height);
-
-	glBegin(GL_POINTS);
+	rows = 0;
+	columns = 0;
 	for (i = 0; i < n_requests; i++) {
 		glyph = _vte_glyph_get(data->cache, requests[i].c);
+		columns += (requests[i].columns * data->cache->width);
 		if ((glyph == NULL) ||
 		    (glyph->width == 0) ||
 		    (glyph->height == 0)) {
 			continue;
 		}
-		w = requests[i].columns * data->cache->width;
-		pad = (w - glyph->width) / 2;
-		for (y = 0; y < glyph->height; y++) {
-			for (x = 0; x < glyph->width; x++) {
-				j = (y * glyph->width + x) *
-				    glyph->bytes_per_pixel;
-				a = glyph->bytes[j] * alpha;
-				if (a == 0) {
-					continue;
-				}
-				glColor4us(r, g, b, a);
-				glVertex2i(requests[i].x + pad + x,
-					   height - (requests[i].y + glyph->skip + y));
-			}
-		}
+		rows = MAX(rows, glyph->skip + glyph->height);
 	}
+
+	_vte_buffer_set_minimum_size(data->buffer, rows * columns * 4);
+	pixels = data->buffer->bytes;
+	memset(pixels, 0, rows * columns * 4);
+	for (i = 0; i < rows * columns; i++) {
+		pixels[i * 4 + 0] = r;
+		pixels[i * 4 + 1] = g;
+		pixels[i * 4 + 2] = b;
+	}
+
+	for (i = j = 0; i < n_requests; i++) {
+		glyph = _vte_glyph_get(data->cache, requests[i].c);
+		w = requests[i].columns * data->cache->width;
+		if ((glyph == NULL) ||
+		    (glyph->width == 0) ||
+		    (glyph->height == 0)) {
+			j += w;
+			continue;
+		}
+		pad = (w - glyph->width) / 2;
+		for (y = 0; y < glyph->height; y++)
+		for (x = 0; x < glyph->width; x++) {
+			src = (y * glyph->width + x) * glyph->bytes_per_pixel;
+			a = glyph->bytes[src];
+			if (a == 0) {
+				continue;
+			}
+			if (alpha != VTE_DRAW_OPAQUE) {
+				a = (a * alpha) >> 8;
+			}
+			if (a == 0) {
+				continue;
+			}
+			dest = (y + glyph->skip) * columns * 4 +
+			       (j + pad + x) * 4 + 3;
+			pixels[dest] = a;
+		}
+		j += w;
+	}
+
+	glRasterPos2i(requests[0].x, requests[0].y);
+	glPixelZoom(1, -1);
+	glDrawPixels(columns, rows, GL_RGBA, GL_UNSIGNED_BYTE, pixels);
+}
+
+static void
+_vte_gl_rectangle(struct _vte_draw *draw,
+		  GLenum type,
+		  gint x, gint y, gint width, gint height,
+		  GdkColor *color, guchar alpha)
+{
+	GdkDisplay *gdisplay;
+	Display *display;
+	struct _vte_gl_data *data;
+
+	data = (struct _vte_gl_data*) draw->impl_data;
+	gdisplay = gdk_display_get_default();
+	display = gdk_x11_display_get_xdisplay(gdisplay);
+
+	glXMakeCurrent(display, data->glwindow, data->context);
+
+	glColor4us(color->red, color->green, color->blue,
+		   (alpha == VTE_DRAW_OPAQUE) ? 0xffff : (alpha << 8));
+	glBegin(type);
+	glVertex2d(x, y);
+	glVertex2d(x + width, y);
+	glVertex2d(x + width, y + height);
+	glVertex2d(x, y + height);
 	glEnd();
 }
 
@@ -475,23 +544,8 @@ _vte_gl_draw_rectangle(struct _vte_draw *draw,
 		       gint x, gint y, gint width, gint height,
 		       GdkColor *color, guchar alpha)
 {
-	struct _vte_gl_data *data;
-	int w, h;
-
-	data = (struct _vte_gl_data*) draw->impl_data;
-
-	glXWaitX();
-
-	gdk_drawable_get_size(data->pixmap, &w, &h);
-
-	glBegin(GL_LINE_LOOP);
-	glColor4us(color->red, color->green, color->blue,
-		   (alpha == VTE_DRAW_OPAQUE) ? 0xffff : (alpha << 8));
-	glVertex2d(x, h - y);
-	glVertex2d(x + width, h - y);
-	glVertex2d(x + width, h - (y + height));
-	glVertex2d(x, h - (y + height));
-	glEnd();
+	_vte_gl_rectangle(draw, GL_LINE_LOOP, x, y, width, height,
+			  color, alpha);
 }
 
 static void
@@ -499,29 +553,15 @@ _vte_gl_fill_rectangle(struct _vte_draw *draw,
 		       gint x, gint y, gint width, gint height,
 		       GdkColor *color, guchar alpha)
 {
-	struct _vte_gl_data *data;
-	int w, h;
-
-	data = (struct _vte_gl_data*) draw->impl_data;
-
-	glXWaitX();
-
-	gdk_drawable_get_size(data->pixmap, &w, &h);
-
-	glBegin(GL_POLYGON);
-	glColor4us(color->red, color->green, color->blue,
-		   (alpha == VTE_DRAW_OPAQUE) ? 0xffff : (alpha << 8));
-	glVertex2d(x, h - y);
-	glVertex2d(x + width, h - y);
-	glVertex2d(x + width, h - (y + height));
-	glVertex2d(x, h - (y + height));
-	glEnd();
+	_vte_gl_rectangle(draw, GL_POLYGON, x, y, width, height,
+			  color, alpha);
 }
 
 static void
 _vte_gl_set_scroll(struct _vte_draw *draw, gint x, gint y)
 {
 	struct _vte_gl_data *data;
+
 	data = (struct _vte_gl_data*) draw->impl_data;
 	data->scrollx = x;
 	data->scrolly = y;
@@ -538,6 +578,7 @@ struct _vte_draw_impl _vte_draw_gl = {
 	_vte_gl_end,
 	_vte_gl_set_background_color,
 	_vte_gl_set_background_image,
+	TRUE,
 	_vte_gl_clear,
 	_vte_gl_set_text_font,
 	_vte_gl_get_text_width,
